﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using RawInputSharp;
using System.Threading;

namespace ttBarebone
{
    public partial class View : Form
    {
        //Internal, quasi-random Smartboard input IDs. Change, if necessary:
        public const int INPUT1ID = 11;
        public const int INPUT2ID = 267;

        //Internal raw mouse coordinates:
        public const int MOUSE1ID = 20;
        public const int MOUSE2ID = 22;

        private Controller ctrl;

        private class Mouse
        {
            public Point pos;
            public bool button;

            public Mouse(Point pos)
            {
                this.button = false;
                this.pos = pos;
            }
        }

        private Mouse mouse1;
        private Mouse mouse2;

        //Multipoint stuff
        public RawMouseInput _rmInput;

        public View()
        {
            InitializeComponent();

            ctrl = new Controller(this);
            InitInput();
        }

        /// <summary>
        /// Hide the mouse cursor and set the mouse positions to the current
        /// cursor position. 
        /// </summary>
        private void InitInput()
        {
            this.KeyDown += new KeyEventHandler(ctrl.KeyDown);
            this.KeyUp += new KeyEventHandler(ctrl.KeyUp);

            Point origin = Cursor.Position;
            mouse1 = new Mouse(new Point(origin.X, origin.Y));
            mouse2 = new Mouse(new Point(origin.X, origin.Y));

            Cursor.Hide();

            _rmInput = new RawMouseInput();
            _rmInput.RegisterForWM_INPUT(this.Handle);
        }

        /// <summary>
        /// Paste text to the console label.
        /// </summary>
        /// <param name="text"></param>
        public void Console(string text)
        {
            this.console.Text = text;
        }

        delegate void SetInputDelegate(int number, int x, int y);

        /// <summary>
        /// Changes the position of the two discs based on the 
        /// Smartboard pointers.
        /// </summary>
        /// <param name="number">Smartboard id of the input</param>
        /// <param name="x">Horizontal position of the input in screen coordinates</param>
        /// <param name="y">Vertical position of the input in screen coordinates</param>
        public void SetInput(int number, int x, int y)
        {
            if (!InvokeRequired)
            {
                switch (number)
                {
                    case INPUT1ID:
                    case MOUSE1ID:
                        if (!input1Image.Visible)
                        {
                            input1Image.Visible = true;
                        }
                        input1Image.Location = new Point(x - (input1Image.Width / 2), y - (input1Image.Height / 2));
                        break;
                    case INPUT2ID:
                    case MOUSE2ID:
                        if (!input2Image.Visible)
                        {
                            input2Image.Visible = true;
                        }
                        input2Image.Location = new Point(x - (input2Image.Width / 2), y - (input2Image.Height / 2));
                        break;
                } 
            }
            else
            {
                Invoke(new SetInputDelegate(SetInput), new object[] { number, x, y });
            }
        }

        /// <summary>
        /// Hide the discs if an input was removed from the table.
        /// </summary>
        /// <param name="number">Smartboard id of the input</param>
        public void ReleaseInput(int number)
        {
            switch (number)
            {
                case INPUT1ID:
                    input1Image.Visible = false;
                    break;
                case INPUT2ID:
                    input2Image.Visible = false;
                    break;
            }
        }

        protected const int WM_INPUT = 0x00FF;
        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case WM_INPUT:
                    //read in new mouse values.
                    _rmInput.UpdateRawMouse(m.LParam);

                    UpdateControls();
                    break;
            }
            base.WndProc(ref m);
        }

        #region Mouse Event Handlers
        /// <summary>
        /// Method is called whenever a mouse event occurs. The mouse events
        /// of up to two raw mice are handled in this method.
        /// </summary>
        private void UpdateControls()
        {
            // find up to two raw mice:
            RawMouse rawm1;
            RawMouse rawm2;

            if(_rmInput.Mice.Count > 0) {
                rawm1 = (RawMouse)_rmInput.Mice[0];
                this.ProcessMouseEvents(rawm1, mouse1, MOUSE1ID);
            }
            if(_rmInput.Mice.Count > 1) {
                rawm2 = (RawMouse)_rmInput.Mice[1];
                this.ProcessMouseEvents(rawm2, mouse2, MOUSE2ID);
            }
        }

        /// <summary>
        /// Moves mouse cursor to new position and triggers mouse events (down, move, up)
        /// </summary>
        /// <param name="rawm"></param>
        /// <param name="mouse"></param>
        /// <param name="mouse"ID></param>
        private void ProcessMouseEvents(RawMouse rawm, Mouse mouse, int mouseID)
        {
            // move mouse position
            if (mouse.Equals(this.mouse1))
            {
                this.mouse1.pos.X += rawm.XDelta;
                this.mouse1.pos.Y += rawm.YDelta;
            }
            else if (mouse.Equals(this.mouse2))
            {
                this.mouse2.pos.X += rawm.XDelta;
                this.mouse2.pos.Y += rawm.YDelta;
            }

            // don't let the mouse position get out of the screen
            if (mouse.pos.X < 0)
            {
                // don't leave window on the left edge
                mouse.pos.X = 0;
            }
            if (mouse.pos.Y < 0)
            {
                // don't leave window on the upper edge
                mouse.pos.Y = 0;
            }
            if (mouse.pos.X > this.Size.Width)
            {
                // don't leave window on the right edge
                mouse.pos.X = this.Size.Width;
            }
            if (mouse.pos.Y > this.Size.Height)
            {
                // don't leave window on the bottom edge
                mouse.pos.Y = this.Size.Height;
            }

            // trigger mouse events
            if (rawm.Buttons[0] && !mouse.button)
            {
                // mouse event 1: mouse down (mouse was not clicked but is now)
                System.Console.WriteLine(mouseID + ": Down Event");
                ctrl.Down(mouse.pos.X, mouse.pos.Y, mouseID);
                mouse.button = true;
            }
            else if (rawm.Buttons[0] && mouse.button)
            {
                // mouse event 2: mouse moved (mouse was clicked and still is)
                System.Console.WriteLine(mouseID + ": Move Event");
                ctrl.Move(mouse.pos.X, mouse.pos.Y, mouseID);
            }
            else if (!rawm.Buttons[0] && mouse.button)
            {
                // mouse event 3: mouse up (mouse was clicked and is not any more)
                System.Console.WriteLine(mouseID + ": Up Event");
                ctrl.Up(mouse.pos.X, mouse.pos.Y, mouseID);
                mouse.button = false;
            }
            else
            {
                // no mouse event: mouse was not clicked and still is not
            }
        }
        #endregion
    }
}